
/* bookmarks.cc */

#include "bookmarks.h"

#include <QPushButton>
#include <QVBoxLayout>

#include <kaboutdata.h>
#include <kaction.h>
#include <kactioncollection.h>
#include <kdebug.h>
#include <kpluginfactory.h>

#include <ktexteditor/document.h>
#include <ktexteditor/markinterface.h>

#include <interfaces/icore.h>
#include <interfaces/idocumentcontroller.h>
// #include <interfaces/iuicontroller.h>

#include <iostream>
#include "trace.h"
#include "traceview.h"

/* ----------------------------------------------------------------------- */

namespace BookmarkModule {

/* ----------------------------------------------------------------------- */

KAboutData aboutData ("bookmarklist",
                      "bookmarklist",
                      ki18n ("Bookmark List"),
                      "0.1",
                      ki18n ("Simple bookmark list"),
                      KAboutData::License_GPL);

K_PLUGIN_FACTORY (BookmarkPluginFactory, registerPlugin <BookmarkPlugin> (); )
K_EXPORT_PLUGIN (BookmarkPluginFactory (aboutData))

/* ----------------------------------------------------------------------- */

QWidget * BookmarkViewFactory::create (QWidget * parent)
{
    return new BookmarkView (m_plugin, parent);
}

Qt::DockWidgetArea BookmarkViewFactory::defaultPosition ()
{
    return Qt::BottomDockWidgetArea;
}

QString BookmarkViewFactory::id () const
{
    return "org.kdevelop.Bookmarks.Info";
}

BookmarkViewFactory::BookmarkViewFactory (BookmarkPlugin * plugin) :
    m_plugin (plugin)
{ }

/* ----------------------------------------------------------------------- */

BookmarkPlugin::BookmarkPlugin (QObject *parent, const QVariantList & args) :
   KDevelop::IPlugin (BookmarkPluginFactory::componentData(), parent),
   m_view_factory (NULL),
   m_trace_factory (NULL)
{
    Q_UNUSED(args)

    std::cout << std::endl;
    std::cout << " **** BOOKMARKS **** " << std::endl;
    std::cout << std::endl;

    setXMLFile ("bookmarklist.rc");

    m_view_factory = new BookmarkViewFactory (this);
    core()->uiController()->addToolView ("Bookmarks", m_view_factory);

    m_trace_factory = new_trace_factory ();
    core()->uiController()->addToolView ("Bookmark Trace", m_trace_factory);

    KActionCollection* actions = actionCollection ();
    KAction *action = actions->addAction ("bookmarks_run");
    action->setText (i18n ("Bookmarks Plugin"));
    // action->setShortcut (Qt::CTRL | Qt::ALT | Qt::Key_7);
    connect (action, SIGNAL (triggered (bool)), this, SLOT (run ()));
}

BookmarkPlugin::~BookmarkPlugin()
{
}

void BookmarkPlugin::unload()
{
    core()->uiController()->removeToolView (m_view_factory);
    core()->uiController()->removeToolView (m_trace_factory);
}

void BookmarkPlugin::run ()
{
    std::cout << std::endl;
    std::cout << " **** BOOKMARKS RUN **** " << std::endl;
    std::cout << std::endl;

    kDebug (8000) << " ---- BOOKMARKS RUN ---- ";
}

/* ----------------------------------------------------------------------- */

BookmarkView::BookmarkView (BookmarkPlugin * plugin, QWidget * parent) :
   QWidget (parent),
   m_plugin (plugin)
{
    setObjectName (i18n ("BookmarkView"));

    setWhatsThis (i18n ("<b>Bookmark View</b><p>"
                        "This window contains an view.</p>" ) );

    setWindowIcon (KIcon ("bookmarks"));

    setWindowTitle (i18n ("Bookmarks" ) );

    QVBoxLayout * layout = new QVBoxLayout(this);

    m_button = new QPushButton ("Bookmarks", this);
    connect (m_button, SIGNAL (clicked ()), this, SLOT (click ()));
    layout->addWidget (m_button, 0);

    m_tree = new QTreeWidget (this);
    connect (m_tree, SIGNAL (itemDoubleClicked (QTreeWidgetItem *, int)),
             this, SLOT (doubleClick (QTreeWidgetItem *, int)));
    #if 1
       connect (m_tree, SIGNAL (itemClicked(QTreeWidgetItem*,int)),
                this, SLOT (doubleClick(QTreeWidgetItem*,int)));
    #endif
    layout->addWidget (m_tree, 1);
}


void BookmarkView::click ()
{
   m_tree->clear ();

   foreach (KDevelop::IDocument * doc, KDevelop::ICore::self()->documentController()->openDocuments())
   {
      KTextEditor::Document* document = doc->textDocument();
      KTextEditor::MarkInterface * mi = dynamic_cast <KTextEditor::MarkInterface*> (document);
      if (mi != NULL)
      {
         QTreeWidgetItem * fileItem = new QTreeWidgetItem (m_tree);
         fileItem->setText (0, doc->url().fileName());
         fileItem->setToolTip (0, doc->url().pathOrUrl());
         fileItem->setIcon (0, KIcon ("folder-blue"));
         QFont f = fileItem->font (0);
         f.setBold (true);
         fileItem->setFont (0, f);
         fileItem->setExpanded (true);

         QHashIterator<int, KTextEditor::Mark*> it = mi->marks();
         while (it.hasNext())
         {
            KTextEditor::Mark * mark = it.next().value();

            QTreeWidgetItem * item = new QTreeWidgetItem (fileItem);
            item->setText (0, "line: " + QString::number (mark->line));

            KTextEditor::MarkInterface::MarkTypes t =
               KTextEditor::MarkInterface::MarkTypes (mark->type);
            item->setIcon (0, mi->markPixmap (t));

            item->setData (0, Qt::DisplayRole, doc->url().pathOrUrl());
            item->setData (1, Qt::DisplayRole, mark->line);
        }
      }
   }

}

void BookmarkView::doubleClick (QTreeWidgetItem * item, int column)
{
   QString fileName = item->data (0, Qt::DisplayRole).toString();
   int lineNumber = item->data (1, Qt::DisplayRole).toInt();
   kDebug() << "jump to" << fileName << "line:" << lineNumber;
   std::cout << std::endl;
   std::cout << " **** BOOKMARKS JUMP **** " << fileName.data() << "line:" << lineNumber << std::endl;
   std::cout << std::endl;

   KUrl url = fileName;
   KTextEditor::Cursor cursor (lineNumber, 0);
   // KDevelop 4.2 // KDevelop::IDocumentController::DocumentActivation mode = KDevelop::IDocumentController::DoNotForceCurrentView;
   KDevelop::IDocumentController::DocumentActivation mode = KDevelop::IDocumentController::DefaultMode;
   KDevelop::ICore::self()->documentController()->openDocument (url, cursor, mode);
}

/* ----------------------------------------------------------------------- */

} // end of namespace

#include "bookmarks.moc"
